package org.simantics.modeling.ui.componentTypeEditor;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.simantics.Simantics;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.layer0.Layer0;
import org.simantics.scl.ui.editor2.StandardSCLModuleEditorInput;
import org.simantics.ui.workbench.editor.AbstractResourceEditorAdapter;
import org.simantics.ui.workbench.editor.EditorAdapter;
import org.simantics.utils.ui.workbench.WorkbenchUtils;

public class SCLModuleEditorAdapter extends AbstractResourceEditorAdapter
        implements EditorAdapter {

    public SCLModuleEditorAdapter() {
        super(Messages.SCLModuleEditorAdapter_SCLModuleEditor, null, 20);
    }
    
    @Override
    public boolean canHandle(ReadGraph g, Object input)
            throws DatabaseException {
        if(input instanceof IStructuredSelection)
            input = ((IStructuredSelection)input).getFirstElement();
        if(!(input instanceof Resource)) {
            if(input instanceof IAdaptable) {
                input = ((IAdaptable)input).getAdapter(Resource.class);
                if(input == null)
                    return false;
            }
            else
                return false;
        }
        Resource resource = (Resource)input;
        Layer0 L0 = Layer0.getInstance(g);
        return g.isInstanceOf(resource, L0.SCLModule);
    }
    
    protected void openEditor(Resource input) throws Exception {
        IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
        if(page == null)
            return;
        Simantics.getSession().asyncRequest(new ReadRequest() {
            @Override
            public void run(ReadGraph graph) throws DatabaseException {
                String uri = graph.getURI(input);
                PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            WorkbenchUtils.openEditor("org.simantics.scl.ui.editor2", //$NON-NLS-1$
                                    new StandardSCLModuleEditorInput(uri));
                        } catch (PartInitException e) {
                            e.printStackTrace();
                        }
                    }
                });
            }
        });
    }

}
